<?php
/**
 * Wicked external API interface.
 *
 * This file defines Wicked's external API interface. Other applications
 * can interact with Wicked through this API.
 *
 * $Horde: wicked/lib/api.php,v 1.25 2008-03-27 04:30:18 bklang Exp $
 *
 * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
 *
 * See the enclosed file COPYING for license information (GPL). If you
 * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
 *
 * @package Wicked
 */

$_services['perms'] = array(
    'args' => array(),
    'type' => 'array'
);

$_services['display'] = array(
    'args' => array('pagename' => 'string'),
    'type' => 'string'
);

$_services['edit'] = array(
    'args' => array('pagename' => 'string',
                    'text' => 'string',
                    'changelog' => 'string',
                    'minorchange' => 'boolean'),
    'type' => 'boolean'
);

$_services['listTemplates'] = array(
    'args' => array(),
    'type' => 'array'
);

$_services['getTemplate'] = array(
    'args' => array('name' => 'string'),
    'type' => 'string'
);

$_services['saveTemplate'] = array(
    'args' => array('name' => 'string',
                    'data' => 'string'),
    'type' => 'boolean'
);

$_services['getPageSource'] = array(
    'args' => array('name' => 'string'),
    'type' => 'text'
);

/**
 * Returns a list of available permissions.
 *
 * @return array  An array describing all available permissions.
 */
function _wicked_perms()
{
    static $perms = array();
    if (!empty($perms)) {
        return $perms;
    }

    require_once dirname(__FILE__) . '/base.php';
    global $wicked;

    $perms['tree']['wicked']['pages'] = array();
    $perms['title']['wicked:pages'] = _("Pages");

    $perms['tree']['wicked']['pages'] = array();
    $perms['title']['wicked:pages'] = _("Pages");

    $perms['tree']['wicked']['pages']['AllPages'] = false;
    $perms['title']['wicked:pages:AllPages'] = 'AllPages';

    $perms['tree']['wicked']['pages']['LeastPopular'] = false;
    $perms['title']['wicked:pages:LeastPopular'] = 'LeastPopular';

    $perms['tree']['wicked']['pages']['MostPopular'] = false;
    $perms['title']['wicked:pages:MostPopular'] = 'MostPopular';

    $perms['tree']['wicked']['pages']['RecentChanges'] = false;
    $perms['title']['wicked:pages:RecentChanges'] = 'RecentChanges';

    foreach ($wicked->getPages() as $pageName) {
        $pageId = $wicked->getPageId($pageName);
        $perms['tree']['wicked']['pages'][$pageId] = false;
        $perms['title']['wicked:pages:' . $pageId] = $pageName;
    }

    ksort($perms['tree']['wicked']['pages']);
    return $perms;
}

/**
 * Returns a rendered wiki page.
 *
 * @param string $pagename Page to display
 *
 * @return array  Page without CSS link
 */
function _wicked_display($pagename)
{
    require_once dirname(__FILE__) . '/base.php';

    $page = Page::getPage($pagename);
    if (is_a($page, 'PEAR_Error')) {
        return $page;
    }

    $GLOBALS['wicked']->logPageView($page->pageName());

    return $page->displayContents(false);
}

/**
 * Updates content of a wiki page. If the page does not exist it is
 * created.
 *
 * @param string $pagename Page to edit
 * @param string $text Page content
 * @param string $changelog Description of the change
 * @param boolean $minorchange True if this is a minor change
 *
 * @return boolean | PEAR_Error True on success, PEAR_Error on failure.
 */
function _wicked_edit($pagename, $text, $changelog = '', $minorchange = false)
{
    require_once dirname(__FILE__) . '/base.php';

    $page = Page::getPage($pagename);
    if (is_a($page, 'PEAR_Error')) {
        return $page;
    }
    if (!$page->allows(WICKED_MODE_EDIT)) {
        return PEAR::RaiseError(sprintf(_("You don't have permission to edit \"%s\"."), $pagename));
    }
    if ($GLOBALS['conf']['wicked']['require_change_log'] &&
        empty($changelog)) {
        return PEAR::raiseError(_("You must provide a change log."));
    }

    $content = $page->getText();
    if (is_a($content, 'PEAR_Error')) {
        // Maybe the page does not exists, if not create it
        if ($GLOBALS['wicked']->pageExists($pagename)) {
            return $content;
        } else {
            return $GLOBALS['wicked']->newPage($pagename, $text);
        }
    }

    if (trim($text) == trim($content)) {
        return PEAR::raiseError(_("No changes made"));
    }

    $result = $page->updateText($text, $changelog, $minorchange);
    if (is_a($result, 'PEAR_Error')) {
        return $result;
    } else {
        return true;
    }
}

/**
 * Get a list of templates provided by Wicked.  A template is any page whose
 * name begins with "Template"
 *
 * @return mixed  Array on success; PEAR_Error on failure
 */
function _wicked_listTemplates()
{
    require_once dirname(__FILE__) . '/base.php';
    global $wicked;
    $templates = $wicked->getMatchingPages('Template', WICKED_PAGE_MATCH_ENDS);
    $list = array(array('category' => _("Wiki Templates"),
                        'templates' => array()));
    foreach ($templates as $page) {
        $list[0]['templates'][] = array('id' => $page['page_name'],
                                        'name' => $page['page_name']);
    }
    return $list;
}

/**
 * Get a template specified by its name.  This is effectively an alias for
 * getPageSource() since Wicked templates are also normal pages.
 * Wicked templates are pages that include "Template" at the beginning of the
 * name.
 *
 * @param string $name  The name of the template to fetch
 *
 * @return mixed        String of template data on success; PEAR_Error on fail
 */
function _wicked_getTemplate($name)
{
    return _wicked_getPageSource($name);
}

/**
 * Get the wiki source of a page specified by its name.
 *
 * @param string $name  The name of the page to fetch
 *
 * @return mixed        String of page data on success; PEAR_Error on fail
 */
function _wicked_getPageSource($pagename)
{
    require_once dirname(__FILE__) . '/base.php';
    global $wicked;

    $page = Page::getPage($pagename);

    if (!$page->allows(WICKED_MODE_CONTENT)) {
        return PEAR::raiseError(_("Permission denied."));
    }

    if (!$page->isValid()) {
        return PEAR::raiseError(_("Invalid page requested."));
    }

    return $page->getText();
}

/**
 * Process a completed template to update the named Wiki page.  This method is
 * basically a passthrough to _wicked_edit()
 *
 * @param string $name   Name of the new or modified page
 * @param string $data   Text content of the populated template
 *
 * @return mixed         True on success; PEAR_Error on failure
 */
function saveTemplate($name, $data)
{
    return _wicked_edit($name, $data, 'Template Auto-fill', false);
}
